@@ -1,24 +1,10 @@
-# -*- coding: utf-8 -*-
-#
-# Copyright (C) 2013-2017 Gauvain Pocentek <gauvain@pocentek.net>
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Wrapper for the GitLab API."""
 
 import os
+import re
 import time
 from typing import Any, cast, Dict, List, Optional, Tuple, TYPE_CHECKING, Union
+from urllib import parse
 
 import requests
 import requests.utils
@@ -65,6 +51,8 @@ class Gitlab:
         user_agent: A custom user agent to use for making HTTP requests.
         retry_transient_errors: Whether to retry after 500, 502, 503, 504
             or 52x responses. Defaults to False.
+        keep_base_url: keep user-provided base URL for pagination if it
+            differs from response headers
     """
 
     def __init__(
@@ -84,6 +72,7 @@ class Gitlab:
         order_by: Optional[str] = None,
         user_agent: str = gitlab.const.USER_AGENT,
         retry_transient_errors: bool = False,
+        keep_base_url: bool = False,
     ) -> None:
 
         self._api_version = str(api_version)
@@ -94,6 +83,7 @@ class Gitlab:
         #: Timeout to use for requests to gitlab server
         self.timeout = timeout
         self.retry_transient_errors = retry_transient_errors
+        self.keep_base_url = keep_base_url
         #: Headers that will be used in request to GitLab
         self.headers = {"User-Agent": user_agent}
 
@@ -277,7 +267,7 @@ class Gitlab:
     @classmethod
     def merge_config(
         cls,
-        options: dict,
+        options: Dict[str, Any],
         gitlab_id: Optional[str] = None,
         config_files: Optional[List[str]] = None,
     ) -> "Gitlab":
@@ -330,7 +320,9 @@ class Gitlab:
         )
 
     @staticmethod
-    def _merge_auth(options: dict, config: gitlab.config.GitlabConfigParser) -> Tuple:
+    def _merge_auth(
+        options: Dict[str, Any], config: gitlab.config.GitlabConfigParser
+    ) -> Tuple[Optional[str], Optional[str], Optional[str]]:
         """
         Return a tuple where at most one of 3 token types ever has a value.
         Since multiple types of tokens may be present in the environment,
@@ -356,13 +348,17 @@ class Gitlab:
         return (None, None, None)
 
     def auth(self) -> None:
-        """Performs an authentication using private token.
+        """Performs an authentication using private token. Warns the user if a
+        potentially misconfigured URL is detected on the client or server side.
 
         The `user` attribute will hold a `gitlab.objects.CurrentUser` object on
         success.
         """
         self.user = self._objects.CurrentUserManager(self).get()
 
+        if hasattr(self.user, "web_url") and hasattr(self.user, "username"):
+            self._check_url(self.user.web_url, path=self.user.username)
+
     def version(self) -> Tuple[str, str]:
         """Returns the version and revision of the gitlab server.
 
@@ -439,6 +435,7 @@ class Gitlab:
         data = self.http_post("/markdown", post_data=post_data, **kwargs)
         if TYPE_CHECKING:
             assert not isinstance(data, requests.Response)
+            assert isinstance(data["html"], str)
         return data["html"]
 
     @gitlab.exceptions.on_http_error(gitlab.exceptions.GitlabLicenseError)
@@ -568,6 +565,36 @@ class Gitlab:
             return path
         return f"{self._url}{path}"
 
+    def _check_url(self, url: Optional[str], *, path: str = "api") -> Optional[str]:
+        """
+        Checks if ``url`` starts with a different base URL from the user-provided base
+        URL and warns the user before returning it. If ``keep_base_url`` is set to
+        ``True``, instead returns the URL massaged to match the user-provided base URL.
+        """
+        if not url or url.startswith(self.url):
+            return url
+
+        match = re.match(rf"(^.*?)/{path}", url)
+        if not match:
+            return url
+
+        base_url = match.group(1)
+        if self.keep_base_url:
+            return url.replace(base_url, f"{self._base_url}")
+
+        utils.warn(
+            message=(
+                f"The base URL in the server response differs from the user-provided "
+                f"base URL ({self.url} -> {base_url}).\nThis is usually caused by a "
+                f"misconfigured base URL on your side or a misconfigured external_url "
+                f"on the server side, and can lead to broken pagination and unexpected "
+                f"behavior. If this is intentional, use `keep_base_url=True` when "
+                f"initializing the Gitlab instance to keep the user-provided base URL."
+            ),
+            category=UserWarning,
+        )
+        return url
+
     @staticmethod
     def _check_redirects(result: requests.Response) -> None:
         # Check the requests history to detect 301/302 redirections.
@@ -669,11 +696,15 @@ class Gitlab:
             GitlabHttpError: When the return code is not 2xx
         """
         query_data = query_data or {}
-        url = self._build_url(path)
+        raw_url = self._build_url(path)
 
-        params: Dict[str, Any] = {}
+        # parse user-provided URL params to ensure we don't add our own duplicates
+        parsed = parse.urlparse(raw_url)
+        params = parse.parse_qs(parsed.query)
         utils.copy_dict(src=query_data, dest=params)
 
+        url = parse.urlunparse(parsed._replace(query=""))
+
         # Deal with kwargs: by default a user uses kwargs to send data to the
         # gitlab server, but this generates problems (python keyword conflicts
         # and python-gitlab/gitlab conflicts).
@@ -808,7 +839,10 @@ class Gitlab:
             and not raw
         ):
             try:
-                return result.json()
+                json_result = result.json()
+                if TYPE_CHECKING:
+                    assert isinstance(json_result, dict)
+                return json_result
             except Exception as e:
                 raise gitlab.exceptions.GitlabParsingError(
                     error_message="Failed to parse the server message"
@@ -818,7 +852,7 @@ class Gitlab:
 
     def http_head(
         self, path: str, query_data: Optional[Dict[str, Any]] = None, **kwargs: Any
-    ) -> requests.structures.CaseInsensitiveDict:
+    ) -> "requests.structures.CaseInsensitiveDict[Any]":
         """Make a HEAD request to the Gitlab server.
 
         Args:
@@ -989,7 +1023,10 @@ class Gitlab:
         )
         try:
             if result.headers.get("Content-Type", None) == "application/json":
-                return result.json()
+                json_result = result.json()
+                if TYPE_CHECKING:
+                    assert isinstance(json_result, dict)
+                return json_result
         except Exception as e:
             raise gitlab.exceptions.GitlabParsingError(
                 error_message="Failed to parse the server message"
@@ -1037,7 +1074,10 @@ class Gitlab:
             **kwargs,
         )
         try:
-            return result.json()
+            json_result = result.json()
+            if TYPE_CHECKING:
+                assert isinstance(json_result, dict)
+            return json_result
         except Exception as e:
             raise gitlab.exceptions.GitlabParsingError(
                 error_message="Failed to parse the server message"
@@ -1113,16 +1153,11 @@ class GitlabList:
         query_data = query_data or {}
         result = self._gl.http_request("get", url, query_data=query_data, **kwargs)
         try:
-            links = result.links
-            if links:
-                next_url = links["next"]["url"]
-            else:
-                next_url = requests.utils.parse_header_links(result.headers["links"])[
-                    0
-                ]["url"]
-            self._next_url = next_url
+            next_url = result.links["next"]["url"]
         except KeyError:
-            self._next_url = None
+            next_url = None
+
+        self._next_url = self._gl._check_url(next_url)
         self._current_page: Optional[str] = result.headers.get("X-Page")
         self._prev_page: Optional[str] = result.headers.get("X-Prev-Page")
         self._next_page: Optional[str] = result.headers.get("X-Next-Page")
